use galactica_content::{Content, FactionHandle, OutfitHandle, ShipHandle, SystemHandle};
use galactica_playeragent::PlayerAgent;
use galactica_system::data::ShipPersonality;
use galactica_system::phys::{PhysImage, PhysSim, PhysSimShipHandle, PhysStepResources};
use galactica_util::timing::Timing;
use nalgebra::Point2;
use std::time::Instant;

pub struct Game {
	// Core game data
	ct: Content,
	systemsim: PhysSim,
	timing: Timing,
	start_instant: Instant,

	// Metadata
	time_scale: f32,
	last_update: Instant,
}

unsafe impl<'a> Send for Game {}

impl<'a> Game {
	pub fn make_player(&mut self) -> PhysSimShipHandle {
		let player = self.systemsim.add_ship(
			&self.ct,
			ShipHandle { index: 0 },
			FactionHandle { index: 0 },
			ShipPersonality::Player,
			Point2::new(0.0, 4000.0),
		);

		let s = self.systemsim.get_ship_mut(&player).unwrap();
		s.add_outfits(
			&self.ct,
			[
				OutfitHandle { index: 0 },
				OutfitHandle { index: 1 },
				OutfitHandle { index: 2 },
			],
		);

		return player;
	}

	pub fn new(ct: Content) -> Self {
		let mut systemsim = PhysSim::new(&ct, SystemHandle { index: 0 });

		let a = systemsim.add_ship(
			&ct,
			ShipHandle { index: 0 },
			FactionHandle { index: 1 },
			ShipPersonality::Point,
			Point2::new(1000.0, 0.0),
		);

		let s = systemsim.get_ship_mut(&a).unwrap();
		s.add_outfits(
			&ct,
			[
				OutfitHandle { index: 0 },
				OutfitHandle { index: 1 },
				OutfitHandle { index: 2 },
			],
		);

		let a = systemsim.add_ship(
			&ct,
			ShipHandle { index: 0 },
			FactionHandle { index: 0 },
			ShipPersonality::Dummy,
			Point2::new(200.0, 2000.0),
		);

		let s = systemsim.get_ship_mut(&a).unwrap();
		s.add_outfits(
			&ct,
			[
				OutfitHandle { index: 0 },
				OutfitHandle { index: 1 },
				OutfitHandle { index: 2 },
			],
		);

		Game {
			ct,
			systemsim,
			timing: Timing::new(),
			start_instant: Instant::now(),
			last_update: Instant::now(),
			time_scale: 1.0,
		}
	}

	pub fn update_player_controls(&mut self, player: &mut PlayerAgent) {
		self.systemsim.update_player_controls(&self.ct, player)
	}

	pub fn step(&mut self, phys_img: &PhysImage) {
		self.timing.start_frame();
		self.systemsim.step(
			PhysStepResources {
				ct: &self.ct,
				t: self.last_update.elapsed().as_secs_f32() * self.time_scale,
				timing: &mut self.timing,
			},
			phys_img,
		);

		self.last_update = Instant::now();
		self.timing.mark_frame();
	}

	pub fn update_image(&self, phys_img: &mut PhysImage) {
		self.systemsim.update_image(phys_img);
	}
}

impl Game {
	pub fn get_timing(&self) -> &Timing {
		&self.timing
	}

	pub fn get_current_time(&self) -> f32 {
		self.start_instant.elapsed().as_secs_f32()
	}
}
